# Programmer : zhuxp
# Date: 
# Last-modified: 01-14-2014, 14:32:37 EST
BLOCKON=3
BLOCKOFF=2
ON=1
OFF=4

DEFAULT_CID=100  ##code machine id  // classify the codw


NORMAL_CODES_ARRAY_HEADER=3721
COMPRESS_CODES_ARRAY_HEADER=9981

BEDGRAPH_CODES_ARRAY_HEADER=4416
